@@ -1,3 +1,29 @@
+
+### Default grid sizes
+
+default.gridsize <- function(d)
+{
+  if (d==1)      gridsize <- 401
+  else if (d==2) gridsize <- rep(151,d)
+  else if (d==3) gridsize <- rep(51, d)
+  else if (d==4) gridsize <- rep(21, d)
+  else gridsize <- NA
+  
+  return(gridsize)
+}
+
+default.bgridsize <- function(d)
+{
+  if (d==1)      gridsize <- 401
+  else if (d==2) gridsize <- rep(151,d)
+  else if (d==3) gridsize <- rep(31, d)
+  else if (d==4) gridsize <- rep(21, d)
+  else  gridsize <- NA
+  
+  return(gridsize)
+}
+
+
 ########################################################################
 ## Linear binning
 ## Courtesy of M Wand 2005
@@ -5,6 +31,7 @@
 ########################################################################
 
 
+
 binning <- function(x, H, h, bgridsize, xmin, xmax, supp=3.7, w)
 {
   x <- as.matrix(x)
@@ -503,3 +530,5 @@ symconv4D.ks <- function(rr, ss, skewflag=rep(1,4) , fftflag=rep(TRUE,2))
 }
 
 
+
+
